* 5 December, 2023  Alex Asher
* Stata code to accompany webinar: Group sequential designs for clinical trials
*******************************************************************************


* gsbounds examples
*******************

 * O'Brien-Fleming efficacy bounds
gsbounds, efficacy(obfleming) nlooks(4) graphbounds

* Pocock efficacy bounds
gsbounds, efficacy(pocock) nlooks(4) graphbounds

* Wang-Tsiatis bounds with efficacy parameter 0 (same as O'Brien-Fleming)
gsbounds, efficacy(wtsiatis(0)) nlooks(4) graphbounds

* Wang-Tsiatis bounds with efficacy parameter 0.5 (same as Pocock)
gsbounds, efficacy(wtsiatis(0.5)) nlooks(4) graphbounds

* Wang-Tsiatis bounds with efficacy parameter 0.25 (between the two)
gsbounds, efficacy(wtsiatis(0.25)) nlooks(4) graphbounds

* Error-spending approximation of O'Brien-Fleming efficacy bounds
gsbounds, efficacy(errobfleming) nlooks(4) graphbounds

* Error-spending approximation of Pocock efficacy bounds
gsbounds, efficacy(errpocock) nlooks(4) graphbounds

* Error-spending Kim-DeMets efficacy bounds with efficacy parameter 1
gsbounds, efficacy(kdemets(1)) nlooks(4) graphbounds

* Error-spending Kim-DeMets efficacy bounds with efficacy parameter 5
gsbounds, efficacy(kdemets(5)) nlooks(4) graphbounds

* Error-spending Hwang-Shih-de Cani efficacy bounds with efficacy parameter 0
gsbounds, efficacy(hsdecani(0)) nlooks(4) graphbounds

* Error-spending Hwang-Shih-de Cani efficacy bounds with efficacy parameter -5
gsbounds, efficacy(hsdecani(-5)) nlooks(4) graphbounds

* One-sided (upper) version of the previous design, with alpha 0.025
gsbounds, efficacy(hsdecani(-5)) nlooks(4) alpha(0.025) upper graphbounds

* Add an O'Brien-Fleming-style futility bound
gsbounds, efficacy(hsdecani(-5)) futility(errobfleming) nlooks(4) alpha(0.025) upper graphbounds



* gsdesign examples
*******************

* Test of two means, four evenly-spaced looks
gsdesign twomeans 50 80, sd1(30) sd2(45) alpha(0.025) power(0.9) onesided efficacy(kdemets(2)) futility(errobfleming) nlooks(4) graphbounds

* With three looks: 50%, 75%, and 100% of the total sample size
gsdesign twomeans 50 80, sd1(30) sd2(45) alpha(0.025) power(0.9) onesided efficacy(kdemets(2)) futility(errobfleming) information(.5 .75 1) graphbounds

* Hypothesis test at first look
use gsdata1, clear
ttest response, by(arm) unequal

* Hypothesis test at second look
use gsdata2, clear
ttest response, by(arm) unequal


* gsdesign usermethod example
*****************************

* sample size evaluator for log-rank test with substantial superiority hypothesis
* (from https://www.stata.com/manuals/adaptgsdesignusermethod.pdf)
capture program drop power_cmd_superlogrank
program power_cmd_superlogrank, rclass
        version 18

        /* parse syntax and check for valid options */
        syntax [anything(name=s1)]  /// P(survival) of control group (optional)
               , ONESIDed           /// one-sided test (required option)
                 [ HRatio(real 0.5) /// hazard ratio under Ha
                   hr0(real 1)      /// hazard ratio under H0
                   WDProb(real 0)   /// P(nonadministrative censoring)
                   NRATio(real 1)   /// ratio of experimental/controls
                   SCHoenfeld       /// use Schoenfeld calculation
                   Alpha(real 0.05) /// significance level
                   Power(real 0.8)  /// power
                   NFRACtional      /// report fractional sample size
                 ]

        /* assume 0% survival if s1 is not specified */
        if (`"`s1'"' != "") {
                confirm number `s1'
                assert (`s1' >= 0) & (`s1' < 1)
        }
        else {
                local s1 = 0
        }
        assert (`hratio'  > 0)
        assert (`hr0'     > 0)
        assert (`nratio'  > 0)
        assert (`wdprob' >= 0) & (`wdprob' < 1)
        assert (`alpha'   > 0) & (`alpha'  < 1)
        assert (`power'   > 0) & (`power'  < 1)

        /* calculate number of failures (events) & fixed-study sample size */
        tempname zalpha zbeta Dratio lhs rhs E s2 prE N N1 N2
        scalar `zalpha' = invnormal(1 - `alpha')
        scalar `zbeta'  = invnormal(`power')
        scalar `Dratio' = `hratio' / `hr0'
        scalar `lhs'    = (`zalpha' + `zbeta')^2 / `nratio'
        if ("`schoenfeld'" != "") {
                /* Schoenfeld calculation */
                scalar `rhs' =  ((`nratio' + 1) / log(`Dratio'))^2
        }
        else {
                /* Freedman calculation */
                scalar `rhs' = ((`nratio' * `Dratio' + 1) / (`Dratio' - 1))^2
        }
        scalar `E'   = `lhs' * `rhs'
        scalar `s2'  = `s1'^`hratio'
        scalar `prE' = 1 - (`s1' + `nratio' * `s2') / (`nratio' + 1)
        scalar `N'   = `E' / (`prE' * (1 - `wdprob'))
        scalar `N1'  = `N' * `nratio' / (`nratio' + 1)
        scalar `N2'  = `N' / (`nratio' + 1)

        if ("`nfractional'" == "") {
                /* round up to a whole number */
                scalar `E'  = ceil(`E')
                scalar `N1' = ceil(`N1')
                scalar `N2' = ceil(`N2')
                scalar `N'  = `N1' + `N2'
        }

        /* return stored results */
        return scalar E           = `E'
        return scalar N           = `N'
        return scalar N1          = `N1'
        return scalar N2          = `N2'
        return scalar hratio      = `hratio'
        return scalar hr0         = `hr0'
        return scalar nratio      = `nratio'
        return scalar s1          = `s1'
        return scalar s2          = `s2'
        return scalar Pr_E        = `prE'
        return scalar Pr_w        = `wdprob'
        return scalar alpha       = `alpha'
        return scalar power       = `power'
        return scalar nfractional = ("`nfractional'" != "")
        return local  direction   = cond(`Dratio' > 1, "upper", "lower")
        return local  endpoint    = "survival"
end

* parser for log-rank test with substantial superiority hypothesis
* (from https://www.stata.com/manuals/adaptgsdesignusermethod.pdf)
capture program drop power_cmd_superlogrank_parse
program power_cmd_superlogrank_parse, sclass
        version 18

        /* parse relevant syntax */
        syntax [anything(name=s1)] ///
               , [WDProb(string)   ///
                  NRATio(string)   ///
                  SCHoenfeld       ///
                  *                /// asterisk (*) captures all other options
                 ]
    
        /* identify parameters to display */
        local diparam hratio hr0
        local grparam HR{sub:a} HR{sub:0}
        if (`"`nratio'"' != "") {
                local diparam `diparam' nratio
                local grparam `grparam' N{sub:2}/N{sub:1}
        }
        if (`"`s1'"' != "") {
                local diparam `diparam' s1 s2 Pr_E
                local grparam `grparam' S{sub:1}(T) S{sub:2}(T) p{sub:E}
        }
        if (`"`wdprob'"' != "") {
                local diparam `diparam' Pr_w
                local grparam `grparam' p{sub:w}
        }

        /* return stored results */
        sreturn clear
        local suptest "Log-rank test for substantial superiority"
        local testtype = cond("`schoenfeld'" == "", "Freedman", "Schoenfeld")
        sreturn local pss_subtitle "`suptest', `testtype' method"
        sreturn local pss_title "for two-sample comparison of survivor functions"
        sreturn local pss_colnames `diparam'
        sreturn local pss_colgrsymbols `grparam'
end

* Design PROFISCOV trial of Sinovac COVID-19 vaccine in Brazil
gsdesign superlogrank 0.99, hratio(0.4) hr0(0.7) wdprob(0.1)     ///
         onesided alpha(0.025) power(0.9) efficacy(hsdecani(-4)) ///
         futility(hsdecani(-4)) information(0.4 1) graphbounds

